/*
* Copyright (C) 2003-2007 Funambol, Inc
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License version 2 as
* published by the Free Software Foundation.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY, TITLE, NONINFRINGEMENT or FITNESS FOR A PARTICULAR
* PURPOSE.  See the GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
* 02111-1307  USA
*/

// PIMSettings.cpp : implementation file
//

#include "stdafx.h"
#include "ui.h"
#include "PIMSettings.h"

#include "ClientUtil.h"
#include "examples/config.h"
#include "base/fscapi.h"
#include "spds/constants.h"
#include "spdm/DMTree.h"
#include "spdm/DMTreeFactory.h"
#include "spdm/ManagementNode.h"

#include "pim/ClientSettings.h"
#include "pim/Maincpp.h"

// CPIMSettings dialog

IMPLEMENT_DYNAMIC(CPIMSettings, CDialog)

CPIMSettings::CPIMSettings(CWnd* pParent /*=NULL*/)
                           : CDialog(CPIMSettings::IDD, pParent) {
}

CPIMSettings::~CPIMSettings() {
}


void CPIMSettings::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
#if defined(WIN32_PLATFORM_WFSP)
    DDX_Control(pDX, IDC_SETTPIM_SPIN1, spinSyncType );
#endif
    DDX_Control(pDX, IDC_PIM_SYNCTYPE, lstSyncType );
    DDX_Control(pDX, IDC_PIM_LINK_ADVANCED, linkAdvanced);
}


BEGIN_MESSAGE_MAP(CPIMSettings, CDialog)
    ON_BN_CLICKED(IDC_PIM_LINK_ADVANCED, &CPIMSettings::OnBnClickedPIMLinkAdvanced)
    ON_WM_CTLCOLOR()
END_MESSAGE_MAP()



BOOL CPIMSettings::OnInitDialog(){
    // SET window title
    CString s1;
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_PIM_SETTINGS);
    SetWindowText(s1);
    CDialog::OnInitDialog();

    // Link for advanced settings is actually hidden.
    linkAdvanced.ShowWindow(FALSE);

#ifdef WIN32_PLATFORM_WFSP
    spinSyncType.SetRange(4,0);	spinSyncType.SetBuddy(&lstSyncType);
#endif

    ZeroMemory(&dlgCommandBar, sizeof(SHMENUBARINFO));
    dlgCommandBar.cbSize = sizeof(SHMENUBARINFO);
    dlgCommandBar.hwndParent = this->GetSafeHwnd();
    dlgCommandBar.dwFlags = SHCMBF_HMENU;
    dlgCommandBar.nToolBarId = IDM_OKCANCEL;
#if defined WIN32_PLATFORM_WFSP
#if (_WIN32_WCE < 0x500)
    dlgCommandBar.nToolBarId = IDR_MENUBAR_OKCANCEL;
#endif
#endif
    dlgCommandBar.hInstRes = getLocalizationUtils()->getLocaleResource();
    dlgCommandBar.nBmpId = 0;
    dlgCommandBar.cBmpImages = 0;
    if(!SHCreateMenuBar(&dlgCommandBar)){
        TRACE(_T("Cannot create command bar!"));
        return FALSE;
    };

    // read string resources
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_SETTCOMM_SYNCTYPE1); lstSyncType.AddString(s1);
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_SETTCOMM_SYNCTYPE2);	lstSyncType.AddString(s1);
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_SETTCOMM_SYNCTYPE3);	lstSyncType.AddString(s1);
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_ADVANCED_SETTINGS);  linkAdvanced.setText(s1);

    LoadStringResources();

    LOGFONT lf;
    memset(&lf, 0, sizeof(LOGFONT));
    HFONT hFont = (HFONT) GetStockObject(SYSTEM_FONT);
    GetObject(hFont, sizeof lf, &lf);
    lf.lfWeight = FW_BOLD;
    lf.lfHeight +=FONT_HEIGHT_OFFSET;
    VERIFY(fontBold.CreateFontIndirect(&lf));

    GetDlgItem(IDC_SETTPIM_STATIC_SYNCTYPE)->SetFont(&fontBold);

    // set ppc header
#if defined(WIN32_PLATFORM_PSPC)
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_PIM_SETTINGS); 
    SetDlgItemText(IDC_HEADER_STATIC, s1);
    GetDlgItem(IDC_HEADER_STATIC)->SetFont(&fontBold);
#endif

    // Read from the registry, get values from 'contact' syncsource
    // as they should be all the same!
    WindowsSyncSourceConfig* ssconf = getRegConfig()->getWinSyncSourceConfig("contact");
    if (!ssconf) {
        TRACE(_T("Source 'contact' not found in registry!"));
        return FALSE;
    }
    CString s3; int pos=0;

    pos = SynctypeReg2UI(ssconf->getSync());
    lstSyncType.SetCurSel(pos);

    // check if another sync is in progress
    if(checkStartSync() != 0){
        lstSyncType.EnableWindow(FALSE);
        #if defined(WIN32_PLATFORM_WFSP)
        spinSyncType.EnableWindow(FALSE);
        #endif
    }

    // fix for wrong title problem, we get & set the title for the main window
    wchar_t title[256];
    AfxGetMainWnd()->GetWindowText(title,256);
    AfxGetMainWnd()->SetWindowText(title);

    lstSyncType.SetFocus();
    return FALSE;
}

void CPIMSettings::OnOK()
{
    if(checkStartSync()){
        CDialog::OnOK();
    }

    int pos = lstSyncType.GetCurSel();
    const char* syncType = SynctypeUI2Reg(pos);

    // Save value in registry for contacts
    WindowsSyncSourceConfig* ssconf = getRegConfig()->getWinSyncSourceConfig("contact");
    if (!ssconf) {
        TRACE(_T("Source 'contact' not found in registry!"));
        return;
    }
    if(strcmp(syncType, ssconf->getSync())){
        ssconf->setSync(syncType);
        getRegConfig()->setDirty(CS_DIRTY_SOURCE_CONTACTS);
    }

    // Save value in registry for calendar
    ssconf = getRegConfig()->getWinSyncSourceConfig("calendar");
    if (!ssconf) {
        TRACE(_T("Source 'calendar' not found in registry!"));
        return;
    }
    if(strcmp(syncType, ssconf->getSync())){
        ssconf->setSync(syncType);
        getRegConfig()->setDirty(CS_DIRTY_SOURCE_CALENDAR);
    }

    // Save value in registry for tasks
    ssconf = getRegConfig()->getWinSyncSourceConfig("task");
    if (!ssconf) {
        TRACE(_T("Source 'task' not found in registry!"));
        return;
    }
    if(strcmp(syncType, ssconf->getSync())){
        ssconf->setSync(syncType);
        getRegConfig()->setDirty(CS_DIRTY_SOURCE_TASKS);
    }
    getRegConfig()->saveDirty();

    CDialog::OnOK();
}



void CPIMSettings::LoadStringResources(){
    CString s1;
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_SYNCTYPE); 
    SetDlgItemText(IDC_SETTPIM_STATIC_SYNCTYPE, s1);
}



void CPIMSettings::OnBnClickedPIMLinkAdvanced() {
    // Not yet implemented, advanced link is not used.
    //CPIMSettingsAdvanced wndContactsAdvanced;
    //INT_PTR result = wndContactsAdvanced.DoModal();
}


// capture pressing of 'Advanced' link
BOOL CPIMSettings::PreTranslateMessage(MSG* pMsg){
    bool bProcessed =false;
    if(pMsg->message == WM_KEYDOWN){
        if(pMsg->wParam == VK_RETURN){
            if(GetFocus()->GetSafeHwnd() == linkAdvanced.GetSafeHwnd()){
                bProcessed = true;
                OnBnClickedPIMLinkAdvanced();
            }
        }
    }

    if(bProcessed)
        return TRUE;
    else
        return CDialog::PreTranslateMessage(pMsg);
}

HBRUSH CPIMSettings::OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor) {
    HBRUSH hbr = CDialog::OnCtlColor(pDC, pWnd, nCtlColor);

    // set ppc header text color
#if defined(WIN32_PLATFORM_PSPC)
    if(pWnd->GetDlgCtrlID() == IDC_HEADER_STATIC) {
        pDC->SetBkMode(TRANSPARENT);
        pDC->SetTextColor(PPC_HEADER_TEXT_COLOR);
    }
#endif

    return hbr;
}