/*
 * Copyright (C) 2003-2007 Funambol, Inc
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY, TITLE, NONINFRINGEMENT or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 * 02111-1307  USA
 */

#ifndef _INCL_UI_CLIENT_UTIL
#define _INCL_UI_CLIENT_UTIL


#define STRING_MAXLENGTH 300
#define LARGE_INT 32000
#define CLOSE_THIS_DIALOG 222

#define ADJUST_WIDTH    1
#define ADJUST_HEIGHT   2
#define ADJUST_BOTH     (ADJUST_WIDTH | ADJUST_HEIGHT)

/**
 * the format in which the last sync time appears in the main window,
 * used by CTime::Format()
 */
//#define LAST_SYNC_TIME_FORMAT TEXT("%b %#d %#I:%M%p")

/**
 * tranforms sync type notation from registry to UI, index pos in listbox
 * @param sSyncType : sync type in Registry format
 * @return the index where this sync type should be in listbox (starts from 0)
 */
int SynctypeReg2UI(const char* sSyncType);

/**
 * tranforms sync type notation from UI to Registry, index pos in listbox
 * @param pos :  position in listbox (starts from 0)
 */
const char* SynctypeUI2Reg(int nPos);

/**
 * checks if the sync is in progress, if yes queries the user if he wants to stop it,
 * if the user chooses yes then the sync is stopped
 * @param : the window handle of a window to we want to attach the timed msgbox, if NULL the msgbox will have no parent
 * @return: -1 if the user chose NO, 0 if the user chose YES, 1 if the sync is not running thus the msgbox wasn't displayed
 */
int EndAppQuery(HWND);

/**
 * checks if a folder exists
 * @param: the folder to be checked
 * @return: TRUE if dir exists, FALSE otherwise
 */
BOOL existsDirectory(wchar_t* dir);

/**
 * Modify the window position and size of the specified offset.
 * For any parameter, 0 means to not change the current value.
 * 
 * @param wnd the window to modify
 * @param x the orizontal offset to add to the position
 * @param y the vertical offset to add to the position
 * @param cx the orizontal offset to add to the size
 * @param cy the vertical offset to add to the size
 *
 * @return non zero on error (the value returned by CWnd::MoveWindow)
 */
void modifyWindow(CWnd *wnd, int x, int y, int cx = 0, int cy = 0);

/**
 * Modify the window size to fit the parent one.
 * 
 * @param wnd the window to modify
 * @param opt can be:
 *          ADJUST_WIDTH
 *          ADJUST_HEIGHT
 *          ADJUST_BOTH   (default)
 * @param lx the orizontal margin from the left border (default to 0)
 * @param rx the orizontal margin from the right border (default to 0)
 * @param ty the vertical margin from the top border (defaults to 0)
 * @param by the vertical margin from the bottom border (defaults to 0)
 *
 * Examples:
 *
 *  Fits exactly the parent window
 *      AdjustToParent(wnd);
 *
 *  Fits width of the parent window, with 10 pixel of margin on each side:
 *      AdjustToParent(wnd, ADJUST_WIDTH, 10, 10);
 *
 *  Fits height of the parent window, with 10 pixel of margin on each side:
 *  AdjustToParent(wnd, ADJUST_HEIGHT, 0, 0, 10, 10);
 *
 */
void adjustToParent(CWnd *wnd, int opt = ADJUST_BOTH,
                    int lx = 0, int rx = 0, int ty = 0, int by = 0);


/**
 * Escape the & char with &&. This is used in label.
 * The Resource will have only & char even if it is not completely
 * true due to a different display for pocket pc and smartphone.
 * The messageBoxes don't need this translation.
 * @param the CString to be escaped
 * @return the CString escaped
 */
CString escapeAmpString(CString s);


/**
 * Draws a line from the point 'start' to the point 'end'.
 *
 * @param dc       the device context where the line should be drawn
 * @param start    the starting point (x,y)
 * @param end      the ending point (x,y)
 * @param color    [OPTIONAL] the color of the line, as a COLORREF (R,G,B) (default = black)
 * @param width    [OPTIONAL] the width of the line in pixels (default = 1)
 * @param penStyle [OPTIONAL] the style of the line, one of: 
 *                 PS_SOLID, PS_DASH, PS_DOT, PS_DASHDOT, PS_DASHDOTDOT
 *                 (default = PS_SOLID)
 */
void drawLine(CDC *dc, CPoint& start, CPoint& end, COLORREF color = (0,0,0), 
              int width = 1, int penStyle = PS_SOLID);

#endif
