/*
 * Copyright (C) 2003-2007 Funambol, Inc
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY, TITLE, NONINFRINGEMENT or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 * 02111-1307  USA
 */

#include "stdafx.h"

#include <cfgmgrapi.h>
#include "Tlhelp32.h"
#include "mail.h"
#include "registry.h"
#include "Winbase.h"  //-- for createProcess
#include "winreg.h"   //-- for registry key reading

#if 1 // Always use portal settings for 1&1
    #define USERNAME    TEXT("")
    #define PASSWORD    TEXT("")
    #define SERVER_URL  TEXT("https://sync.mailxchange.de:8443/funambol/ds")
    #define IS_PORTAL   TEXT("1")
    #define COMPRESSION TEXT("1")
#else
    #define USERNAME    TEXT("guest")
    #define PASSWORD    TEXT("guest")
    #define SERVER_URL  TEXT("http://<server name>:<port>/funambol/ds")
    #define IS_PORTAL   TEXT("0")
    #define COMPRESSION TEXT("1")
#endif




wchar_t* getMyDocumentFolder();

const wchar_t* getAppName(){

    return PROGRAM_DIR;
}
wchar_t* rpath = NULL;
const wchar_t* getFullReg(wchar_t* val){

    if (rpath){
        delete[] rpath;
        rpath = NULL;
    }
    int dim = wcslen(val) + wcslen(getAppName()) + 1;
    rpath = new wchar_t[dim];
    wsprintf(rpath,val,getAppName());
    return rpath;
}

static size_t getLenEncoding(const WCHAR* s)
{
    if (!s)
        return 0;

    int len = wcslen(s);

    if (!len)
        return 0;

    long k = WideCharToMultiByte (CP_UTF8, 0, s, len, 0, 0, 0, 0);

    return (k != 0) ? (long)k : -1;
}

char* toMultibyte(const WCHAR *wc)
{
    if (!wc) {
        return 0;
    }

    char *ret;
    size_t wlen = wcslen(wc);

    if (!wlen) {
        ret = new char[1];
        ret[0] = 0;
        return ret;
    }

    size_t blen = getLenEncoding(wc);

    if(blen <= 0)
        return 0;

    ret = new char[blen+1];

    blen = WideCharToMultiByte(CP_UTF8, 0, wc, wlen, ret, blen, 0, 0);
    ret[blen] = 0;

    return ret;
}





void deleteSyncmlTransportRegistry() {
    RegDeleteKey(HKEY_LOCAL_MACHINE, TEXT("SOFTWARE\\Microsoft\\Inbox\\Svc\\SyncML"));
}

void deleteAllRegistry() {
    RegDeleteKey(HKEY_LOCAL_MACHINE,getFullReg( TEXT("SOFTWARE\\%s")));
}

void deleteSyncmlTree() {
    RegDeleteKey(HKEY_LOCAL_MACHINE,getFullReg( TEXT("SOFTWARE\\%s\\SyncClient\\spds\\syncml")));
}

void deleteVersion30RegistryValues() {
    HKEY key = NULL;
    DWORD res;
    bool ret = false;

    res = RegOpenKeyEx(
            HKEY_LOCAL_MACHINE,
            getFullReg( TEXT("Software\\%s\\SyncClient")),
            0,
            NULL,
            &key
            );

    if (res == ERROR_SUCCESS) {
        RegDeleteValue(key, TEXT("logLevel"));
    }

    if (key != 0) {
        RegCloseKey(key);
    }

}

void getJustInsertedRegistry() {


    wchar_t* value = NULL;
    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient")), TEXT("path"));
    setPropertyValue(TEXT("SyncClient"), TEXT("path"),    value, true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient")), TEXT("pluginVersion"));
    setPropertyValue(TEXT("SyncClient"), TEXT("pluginVersion"),    value, true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\syncml\\Conn")), TEXT("userAgent"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Conn"), TEXT("userAgent"),    value, true);
    if (value) { delete [] value; value = NULL; }
    
    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\syncml\\DevDetail")), TEXT("swv"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\DevDetail"), TEXT("swv"),    value, true);
    if (value) { delete [] value; value = NULL; }


}

void createSyncmlTransportRegistry()
{
    HKEY key = NULL;
    DWORD res;
    //wchar_t port [256];
    //wchar_t name [256];
    //wchar_t dll [256];
    //wcscpy(port, TEXT("SOFTWARE\\Microsoft\\Inbox\\Svc\\%s\\Port"));
    //wcscpy(name, TEXT("SOFTWARE\\Microsoft\\Inbox\\Svc\\%s\\Name"));
    //wcscpy(dll, TEXT("SOFTWARE\\Microsoft\\Inbox\\Svc\\%s\\DLL"));

    const wchar_t portValue[] = TEXT("0");
    const wchar_t nameValue[] = TEXT("SyncML"); 
    // = new wchar_t[wcslen(getFullReg( TEXT("%s")))];
    // wcscpy(nameValue, getFullReg( TEXT("%s")));
    const wchar_t DLLValue[] = TEXT("syncmltransport.dll");

    RegCreateKeyEx(
            HKEY_LOCAL_MACHINE,
            getFullReg( TEXT("SOFTWARE\\Microsoft\\Inbox\\Svc\\SyncML")),
            0,
            NULL,
            REG_OPTION_NON_VOLATILE,
            KEY_ALL_ACCESS,
            NULL,
            &key,
            &res
            );

    if (key == 0) {
       goto finally;
    }

    RegSetValueEx( key, TEXT("port"),   0,
            REG_SZ,  // we currently support only strings
            (UCHAR*)portValue, (wcslen(portValue)+1)*sizeof(WCHAR) );

    RegSetValueEx(key, TEXT("Name"), 0,
            REG_SZ,  // we currently support only strings
            (UCHAR*)nameValue,  (wcslen(nameValue)+1)*sizeof(WCHAR) );

    RegSetValueEx(key, TEXT("DLL"),  0,
            REG_SZ,  // we currently support only strings
            (UCHAR*)DLLValue,(wcslen(DLLValue)+1)*sizeof(WCHAR)  );

    //
    // To create the UserCannotCreate property. Currently not supported
    //
    /*
    RegSetValueEx(
            key,
            TEXT("UserCannotCreate"),
            0,
            REG_DWORD,  // we currently support only strings
            (UCHAR*)TEXT("1"),
            (wcslen(TEXT("1"))+1)*sizeof(wchar_t)
            );
    */
    RegFlushKey(key);

    finally:
    if (key != 0) {
        RegCloseKey(key);
    }
}

/*
* backup sources parameter not present in previous version
*/
bool restoreExtraSources() {
    wchar_t* value = NULL;
    // Contacts
    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\sources\\contacts")), TEXT("schedule"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\contacts"), TEXT("schedule"),  value                                );
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\sources\\contacts")), TEXT("encryption"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\contacts"), TEXT("encryption"),  value                                );
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\sources\\appointments")), TEXT("schedule"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\appointments"), TEXT("schedule"),  value                              );
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\sources\\appointments")), TEXT("encryption"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\appointments"), TEXT("encryption"),  value                            );
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\sources\\tasks")), TEXT("schedule"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\tasks"), TEXT("schedule"),  value                              );
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\sources\\tasks")), TEXT("encryption"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\tasks"), TEXT("encryption"),  value                            );
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\sources\\briefcase")), TEXT("schedule"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\briefcase"), TEXT("schedule"),  value                              );
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\sources\\briefcase")), TEXT("encryption"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\briefcase"), TEXT("encryption"),  value                            );
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\sources\\notes")), TEXT("schedule"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\notes"), TEXT("schedule"),  value                                );
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\sources\\notes")), TEXT("encryption"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\notes"), TEXT("encryption"),  value                                );
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\sources\\mails")), TEXT("schedule"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\mails"), TEXT("schedule"),  value                                );
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\sources\\mails")), TEXT("encryption"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\mails"), TEXT("encryption"),  value                                );
    if (value) { delete [] value; value = NULL; }

    return true;
}

bool restoreSources() {
     wchar_t* value = NULL;
    // Contacts
    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\sources\\contacts")), TEXT("sync"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\contacts"), TEXT("sync"),  value                                );
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\sources\\contacts")), TEXT("last"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\contacts"), TEXT("last"),  value                                );
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\sources\\contacts")), TEXT("name"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\contacts"), TEXT("name"),  value                                );
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\sources\\contacts")), TEXT("type"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\contacts"), TEXT("type"),  value                                );
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\sources\\contacts")), TEXT("syncModes"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\contacts"), TEXT("syncModes"),  value                           );
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\sources\\contacts")), TEXT("uri"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\contacts"), TEXT("uri"),  value                                 );
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\sources\\contacts")), TEXT("useSIF"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\contacts"), TEXT("useSIF"),  value                              );
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\sources\\contacts")), TEXT("encoding"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\contacts"), TEXT("encoding"),  value                            );
    if (value) { delete [] value; value = NULL; }


    // Appointment

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\sources\\appointments")), TEXT("sync"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\appointments"), TEXT("sync"),  value                                );
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\sources\\appointments")), TEXT("last"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\appointments"), TEXT("last"),  value                                );
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\sources\\appointments")), TEXT("name"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\appointments"), TEXT("name"),  value                                );
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\sources\\appointments")), TEXT("type"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\appointments"), TEXT("type"),  value                                );
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\sources\\appointments")), TEXT("syncModes"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\appointments"), TEXT("syncModes"),  value                           );
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\sources\\appointments")), TEXT("uri"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\appointments"), TEXT("uri"),  value                                 );
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\sources\\appointments")), TEXT("useSIF"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\appointments"), TEXT("useSIF"),  value                              );
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\sources\\appointments")), TEXT("encoding"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\appointments"), TEXT("encoding"),  value                            );
    if (value) { delete [] value; value = NULL; }

    // Tasks

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\sources\\tasks")), TEXT("sync"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\tasks"), TEXT("sync"),  value                                );
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\sources\\tasks")), TEXT("last"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\tasks"), TEXT("last"),  value                                );
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\sources\\tasks")), TEXT("name"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\tasks"), TEXT("name"),  value                                );
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\sources\\tasks")), TEXT("type"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\tasks"), TEXT("type"),  value                                );
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\sources\\tasks")), TEXT("syncModes"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\tasks"), TEXT("syncModes"),  value                           );
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\sources\\tasks")), TEXT("uri"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\tasks"), TEXT("uri"),  value                                 );
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\sources\\tasks")), TEXT("useSIF"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\tasks"), TEXT("useSIF"),  value                              );
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\sources\\tasks")), TEXT("encoding"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\tasks"), TEXT("encoding"),  value                            );
    if (value) { delete [] value; value = NULL; }

    // briefcase

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\sources\\briefcase")), TEXT("sync"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\briefcase"), TEXT("sync"),  value                                );
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\sources\\briefcase")), TEXT("last"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\briefcase"), TEXT("last"),  value                                );
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\sources\\briefcase")), TEXT("name"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\briefcase"), TEXT("name"),  value                                );
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\sources\\briefcase")), TEXT("type"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\briefcase"), TEXT("type"),  value                                );
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\sources\\briefcase")), TEXT("syncModes"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\briefcase"), TEXT("syncModes"),  value                           );
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\sources\\briefcase")), TEXT("uri"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\briefcase"), TEXT("uri"),  value                                 );
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\sources\\briefcase")), TEXT("useSIF"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\briefcase"), TEXT("useSIF"),  value                              );
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\sources\\briefcase")), TEXT("encoding"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\briefcase"), TEXT("encoding"),  value                            );
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\sources\\briefcase")), TEXT("dir"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\briefcase"), TEXT("dir"),  value                            );
    if (value) { delete [] value; value = NULL; }

    // notes

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\sources\\notes")), TEXT("sync"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\notes"), TEXT("sync"),  value                                );
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\sources\\notes")), TEXT("last"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\notes"), TEXT("last"),  value                                );
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\sources\\notes")), TEXT("name"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\notes"), TEXT("name"),  value                                );
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\sources\\notes")), TEXT("type"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\notes"), TEXT("type"),  value                                );
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\sources\\notes")), TEXT("syncModes"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\notes"), TEXT("syncModes"),  value                           );
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\sources\\notes")), TEXT("uri"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\notes"), TEXT("uri"),  value                                 );
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\sources\\notes")), TEXT("useSIF"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\notes"), TEXT("useSIF"),  value                              );
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\sources\\notes")), TEXT("encoding"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\notes"), TEXT("encoding"),  value                            );
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\sources\\notes")), TEXT("dir"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\notes"), TEXT("dir"),  value                            );
    if (value) { delete [] value; value = NULL; }

    // Mail

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\sources\\mails")), TEXT("sync"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\mails"), TEXT("sync"),  value                                );
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\sources\\mails")), TEXT("last"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\mails"), TEXT("last"),  value                                );
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\sources\\mails")), TEXT("name"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\mails"), TEXT("name"),  value                                );
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\sources\\mails")), TEXT("type"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\mails"), TEXT("type"),  value                                );
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\sources\\mails")), TEXT("syncModes"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\mails"), TEXT("syncModes"),  value                                );
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\sources\\mails")), TEXT("uri"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\mails"), TEXT("uri"),  value                                );
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\sources\\mails")), TEXT("useSIF"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\mails"), TEXT("useSIF"),  value                                );
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\sources\\mails")), TEXT("encoding"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\mails"), TEXT("encoding"),  value                                );
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\sources\\mails")), TEXT("Inbox"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\mails"), TEXT("Inbox"),  value                                );
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\sources\\mails")), TEXT("Outbox"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\mails"), TEXT("Outbox"),  value                                );
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\sources\\mails")), TEXT("Trash"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\mails"), TEXT("Trash"),  TEXT("0")                                );
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\sources\\mails")), TEXT("Sent"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\mails"), TEXT("Sent"),  TEXT("0")                                );
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\sources\\mails")), TEXT("Draft"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\mails"), TEXT("Draft"),  TEXT("0")                                );
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\sources\\mails")), TEXT("attachSize"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\mails"), TEXT("attachSize"),  value                                );
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\sources\\mails")), TEXT("bodySize"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\mails"), TEXT("bodySize"),  value                                );
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\sources\\mails")), TEXT("downloadAge"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\mails"), TEXT("downloadAge"),  value                                );
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\sources\\mails")), TEXT("mailMaxMsgSize"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\mails"), TEXT("mailMaxMsgSize"),  value                                );
    if (value) { delete [] value; value = NULL; }

    return true;
}

bool backupExtraSources() {
      wchar_t* value = NULL;
    // Contacts
    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\sources\\contacts")), TEXT("schedule"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\contacts"), TEXT("schedule"),  value                                , true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\sources\\contacts")), TEXT("encryption"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\contacts"), TEXT("encryption"),  value                                , true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\sources\\appointments")), TEXT("schedule"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\appointments"), TEXT("schedule"),  value                              , true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\sources\\appointments")), TEXT("encryption"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\appointments"), TEXT("encryption"),  value                            , true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\sources\\tasks")), TEXT("schedule"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\tasks"), TEXT("schedule"),  value                              , true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\sources\\tasks")), TEXT("encryption"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\tasks"), TEXT("encryption"),  value                            , true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\sources\\briefcase")), TEXT("schedule"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\briefcase"), TEXT("schedule"),  value                              , true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\sources\\briefcase")), TEXT("encryption"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\briefcase"), TEXT("encryption"),  value                            , true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\sources\\notes")), TEXT("schedule"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\notes"), TEXT("schedule"),  value                                , true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\sources\\notes")), TEXT("encryption"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\notes"), TEXT("encryption"),  value                                , true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\sources\\mails")), TEXT("schedule"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\mails"), TEXT("schedule"),  value                                , true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\sources\\mails")), TEXT("encryption"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\mails"), TEXT("encryption"),  value                                , true);
    if (value) { delete [] value; value = NULL; }

    return true;
}

/*
* In the backup procedure the syncmode are overwritten with the new values
*/

bool backupSources() {

    wchar_t* value = NULL;
    // Contacts
    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\sources\\contacts")), TEXT("sync"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\contacts"), TEXT("sync"),  value                                , true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\sources\\contacts")), TEXT("last"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\contacts"), TEXT("last"),  value                                , true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\sources\\contacts")), TEXT("name"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\contacts"), TEXT("name"),  value                                , true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\sources\\contacts")), TEXT("type"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\contacts"), TEXT("type"),  value                                , true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\sources\\contacts")), TEXT("syncModes"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\contacts"), 
        TEXT("syncModes"),  TEXT("slow,two-way,one-way-server,one-way-client,refresh-server,refresh-client"), true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\sources\\contacts")), TEXT("uri"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\contacts"), TEXT("uri"),  value                                 , true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\sources\\contacts")), TEXT("useSIF"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\contacts"), TEXT("useSIF"),  value                              , true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\sources\\contacts")), TEXT("encoding"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\contacts"), TEXT("encoding"),  value                            , true);
    if (value) { delete [] value; value = NULL; }


    // Appointment

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\sources\\appointments")), TEXT("sync"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\appointments"), TEXT("sync"),  value                                , true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\sources\\appointments")), TEXT("last"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\appointments"), TEXT("last"),  value                                , true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\sources\\appointments")), TEXT("name"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\appointments"), TEXT("name"),  value                                , true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\sources\\appointments")), TEXT("type"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\appointments"), TEXT("type"),  value                                , true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\sources\\appointments")), TEXT("syncModes"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\appointments"), 
        TEXT("syncModes"),  TEXT("slow,two-way,one-way-server,one-way-client,refresh-server,refresh-client"), true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\sources\\appointments")), TEXT("uri"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\appointments"), TEXT("uri"),  value                                 , true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\sources\\appointments")), TEXT("useSIF"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\appointments"), TEXT("useSIF"),  value                              , true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\sources\\appointments")), TEXT("encoding"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\appointments"), TEXT("encoding"),  value                            , true);
    if (value) { delete [] value; value = NULL; }

    // Tasks

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\sources\\tasks")), TEXT("sync"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\tasks"), TEXT("sync"),  value                                , true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\sources\\tasks")), TEXT("last"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\tasks"), TEXT("last"),  value                                , true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\sources\\tasks")), TEXT("name"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\tasks"), TEXT("name"),  value                                , true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\sources\\tasks")), TEXT("type"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\tasks"), TEXT("type"),  value                                , true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\sources\\tasks")), TEXT("syncModes"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\tasks"), 
        TEXT("syncModes"),  TEXT("slow,two-way,one-way-server,one-way-client,refresh-server,refresh-client"), true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\sources\\tasks")), TEXT("uri"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\tasks"), TEXT("uri"),  value                                 , true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\sources\\tasks")), TEXT("useSIF"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\tasks"), TEXT("useSIF"),  value                              , true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\sources\\tasks")), TEXT("encoding"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\tasks"), TEXT("encoding"),  value                            , true);
    if (value) { delete [] value; value = NULL; }

    // briefcase

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\sources\\briefcase")), TEXT("sync"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\briefcase"), TEXT("sync"),  value                                , true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\sources\\briefcase")), TEXT("last"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\briefcase"), TEXT("last"),  value                                , true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\sources\\briefcase")), TEXT("name"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\briefcase"), TEXT("name"),  value                                , true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\sources\\briefcase")), TEXT("type"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\briefcase"), TEXT("type"),  value                                , true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\sources\\briefcase")), TEXT("syncModes"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\briefcase"), 
        TEXT("syncModes"),  TEXT("slow,two-way,one-way-server,one-way-client,refresh-server,refresh-client"), true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\sources\\briefcase")), TEXT("uri"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\briefcase"), TEXT("uri"),  value                                 , true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\sources\\briefcase")), TEXT("dir"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\briefcase"), TEXT("dir"),  value                              , true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\sources\\briefcase")), TEXT("encoding"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\briefcase"), TEXT("encoding"),  value                            , true);
    if (value) { delete [] value; value = NULL; }
    // not put encryption and schedule. these values will be set extra

    // notes

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\sources\\notes")), TEXT("sync"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\notes"), TEXT("sync"),  value                                , true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\sources\\notes")), TEXT("last"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\notes"), TEXT("last"),  value                                , true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\sources\\notes")), TEXT("name"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\notes"), TEXT("name"),  value                                , true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\sources\\notes")), TEXT("type"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\notes"), TEXT("type"),  value                                , true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\sources\\notes")), TEXT("syncModes"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\notes"), 
        TEXT("syncModes"),  TEXT("slow,two-way,one-way-server,one-way-client,refresh-server,refresh-client"), true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\sources\\notes")), TEXT("uri"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\notes"), TEXT("uri"),  value                                 , true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\sources\\notes")), TEXT("useSIF"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\notes"), TEXT("useSIF"),  value                              , true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\sources\\notes")), TEXT("encoding"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\notes"), TEXT("encoding"),  value                            , true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\sources\\notes")), TEXT("dir"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\notes"), TEXT("dir"),  value                            , true);
    if (value) { delete [] value; value = NULL; }

    // Mail
    // it needs because in 3.0 the inbox and outbox can be synched separately.
    // not true in v6 version. So if the sync is two-way the inbox and outbox are checked
    bool isTwoWay = false;

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\sources\\mails")), TEXT("sync"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\mails"), TEXT("sync"),  value                                , true);
    if (value && wcscmp(value, TEXT("two-way")) != NULL)
        isTwoWay = true;
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\sources\\mails")), TEXT("last"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\mails"), TEXT("last"),  value                                , true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\sources\\mails")), TEXT("name"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\mails"), TEXT("name"),  value                                , true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\sources\\mails")), TEXT("type"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\mails"), TEXT("type"),  value                                , true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\sources\\mails")), TEXT("syncModes"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\mails"), 
        TEXT("syncModes"),  TEXT("slow,two-way,refresh-server"), true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\sources\\mails")), TEXT("uri"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\mails"), TEXT("uri"),  value                                , true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\sources\\mails")), TEXT("useSIF"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\mails"), TEXT("useSIF"),  value                                , true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\sources\\mails")), TEXT("encoding"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\mails"), TEXT("encoding"),  value                                , true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\sources\\mails")), TEXT("Inbox"));
    if (isTwoWay)
        wcscpy(value, TEXT("1"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\mails"), TEXT("Inbox"),  value                                , true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\sources\\mails")), TEXT("Outbox"));
    if (isTwoWay)
        wcscpy(value, TEXT("1"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\mails"), TEXT("Outbox"),  value                                , true);
    if (value) { delete [] value; value = NULL; }

    // trash not syncable in v 6.0
    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\sources\\mails")), TEXT("Trash"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\mails"), TEXT("Trash"),  TEXT("0")                                , true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\sources\\mails")), TEXT("Sent"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\mails"), TEXT("Sent"),  TEXT("0")                                , true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\sources\\mails")), TEXT("Draft"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\mails"), TEXT("Draft"),  TEXT("0")                                , true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\sources\\mails")), TEXT("attachSize"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\mails"), TEXT("attachSize"),  value                                , true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\sources\\mails")), TEXT("bodySize"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\mails"), TEXT("bodySize"),  value                                , true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\sources\\mails")), TEXT("downloadAge"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\mails"), TEXT("downloadAge"),  value                                , true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\sources\\mails")), TEXT("mailMaxMsgSize"));
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\mails"), TEXT("mailMaxMsgSize"),  value                                , true);
    if (value) { delete [] value; value = NULL; }

    return true;
}


bool backupRegistryVersion() {

    createAllRegistryBackup();

    wchar_t* value = NULL;
    /*
    * first of all check if the current installing version is the same type of the installed one
    * If the installing is portal and the installed not, then reset the username and password and
    * server url with the default. The same if the installed is portal and the installing not.
    * If the type is the same then keep the previous configuration
    */
    bool keepThePreviousConfig = true;
    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient")), TEXT("isPortal"));
    if (value && wcscmp(value, IS_PORTAL) != 0) {  //IS_PORTAL is just put in the bkp registry
        keepThePreviousConfig = false;
    }
    if (value) { delete [] value; value = NULL; }
    // Auth
    if (keepThePreviousConfig) {
        value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\syncml\\Auth")), TEXT("username"));
        setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Auth"), TEXT("username"),    value, true);
        if (value) { delete [] value; value = NULL; }

        value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\syncml\\Auth")), TEXT("password"));
        setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Auth"), TEXT("password"),    value, true);
        if (value) { delete [] value; value = NULL; }
    }
    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\syncml\\Auth")), TEXT("clientNonce"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Auth"), TEXT("clientNonce"),    value, true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\syncml\\Auth")), TEXT("serverNonce"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Auth"), TEXT("serverNonce"),    value, true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\syncml\\Auth")), TEXT("serverID"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Auth"), TEXT("serverID"),    value, true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\syncml\\Auth")), TEXT("serverPWD"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Auth"), TEXT("serverPWD"),    value, true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\syncml\\Auth")), TEXT("isServerAuthRequired"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Auth"), TEXT("isServerAuthRequired"),    value, true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\syncml\\Auth")), TEXT("clientAuthType"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Auth"), TEXT("clientAuthType"),    value, true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\syncml\\Auth")), TEXT("serverAuthType"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Auth"), TEXT("serverAuthType"),    value, true);
    if (value) { delete [] value; value = NULL; }

    // Conn
    if (keepThePreviousConfig) {
        value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\syncml\\Conn")), TEXT("syncUrl"));
        setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Conn"), TEXT("syncUrl"),    value, true);
        if (value) { delete [] value; value = NULL; }
    }
    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\syncml\\Conn")), TEXT("proxyHost"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Conn"), TEXT("proxyHost"),    value, true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\syncml\\Conn")), TEXT("proxyPort"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Conn"), TEXT("proxyPort"),    value, true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\syncml\\Conn")), TEXT("useProxy"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Conn"), TEXT("useProxy"),    value, true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\syncml\\Conn")), TEXT("proxyUsername"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Conn"), TEXT("proxyUsername"),    value, true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\syncml\\Conn")), TEXT("proxyPassword"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Conn"), TEXT("proxyPassword"),    value, true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\syncml\\Conn")), TEXT("checkConn"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Conn"), TEXT("checkConn"),    value, true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\syncml\\Conn")), TEXT("responseTimeout"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Conn"), TEXT("responseTimeout"),    value, true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\syncml\\Conn")), TEXT("readBufferSize"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Conn"), TEXT("readBufferSize"),    value, true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\syncml\\Conn")), TEXT("enableCompression"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Conn"), TEXT("enableCompression"),    value, true);
    if (value) { delete [] value; value = NULL; }


    // DevInfo

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\syncml\\DevInfo")), TEXT("devID"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\DevInfo"), TEXT("devID"),    value, true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\syncml\\DevInfo")), TEXT("dsV"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\DevInfo"), TEXT("dsV"),    value, true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\syncml\\DevInfo")), TEXT("man"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\DevInfo"), TEXT("man"),    value, true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\syncml\\DevInfo")), TEXT("mod"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\DevInfo"), TEXT("mod"),    value, true);
    if (value) { delete [] value; value = NULL; }

    //DevDetail
    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\syncml\\DevDetail")), TEXT("devType"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\DevDetail"), TEXT("devType"),    value, true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\syncml\\DevDetail")), TEXT("fwv"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\DevDetail"), TEXT("fwv"),    value, true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\syncml\\DevDetail")), TEXT("hwv"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\DevDetail"), TEXT("hwv"),    value, true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\syncml\\DevDetail")), TEXT("swv"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\DevDetail"), TEXT("swv"),    value, true);
    if (value) { delete [] value; value = NULL; }
    
    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\syncml\\DevDetail")), TEXT("oem"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\DevDetail"), TEXT("oem"),    value, true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\syncml\\DevDetail")), TEXT("loSupport"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\DevDetail"), TEXT("loSupport"),    value, true);
    if (value) { delete [] value; value = NULL; }

    // Ext
    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\syncml\\Ext")), TEXT("begin"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Ext"), TEXT("begin"),    value, true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\syncml\\Ext")), TEXT("end"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Ext"), TEXT("end"),    value, true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\syncml\\Ext")), TEXT("devInfHash"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Ext"), TEXT("devInfHash"),    value, true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\syncml\\Ext")), TEXT("firstTimeSyncMode"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Ext"), TEXT("firstTimeSyncMode"),    value, true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\syncml\\Ext")), TEXT("logLevel"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Ext"), TEXT("logLevel"),    value, true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\syncml\\Ext")), TEXT("maxObjSize"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Ext"), TEXT("maxObjSize"),    value, true);
    if (value) { delete [] value; value = NULL; }


    // get the maxMsgSize from the installation procedure and not the previous one
    //value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\syncml\\Ext"), TEXT("maxMsgSize"));
    //setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Ext"), TEXT("maxMsgSize"),    value, true);
    //if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\syncml\\Ext")), TEXT("nocSupport"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Ext"), TEXT("nocSupport"),    value, true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\syncml\\Ext")), TEXT("utc"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Ext"), TEXT("utc"),    value, true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\syncml\\Ext")), TEXT("verDTD"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Ext"), TEXT("verDTD"),    value, true);
    if (value) { delete [] value; value = NULL; }


    // SyncClient

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient")), TEXT("path"));
    setPropertyValue(TEXT("SyncClient"), TEXT("path"),    value, true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient")), TEXT("push"));
    setPropertyValue(TEXT("SyncClient"), TEXT("push"),    value, true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient")), TEXT("polling"));
    setPropertyValue(TEXT("SyncClient"), TEXT("polling"),    value, true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient")), TEXT("svrNotified"));
    setPropertyValue(TEXT("SyncClient"), TEXT("svrNotified"),    value, true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient")), TEXT("pushPort"));
    setPropertyValue(TEXT("SyncClient"), TEXT("pushPort"),    value, true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient")), TEXT("sms"));
    setPropertyValue(TEXT("SyncClient"), TEXT("sms"),    value, true);
    if (value) { delete [] value; value = NULL; }
    
    /****** registry ctp ************/
    
    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient")), TEXT("queuePush"));
    setPropertyValue(TEXT("SyncClient"), TEXT("queuePush"),    value, true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient")), TEXT("ctpRetry"));
    setPropertyValue(TEXT("SyncClient"), TEXT("ctpRetry"),    value, true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient")), TEXT("maxCtpRetry"));
    setPropertyValue(TEXT("SyncClient"), TEXT("maxCtpRetry"),    value, true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient")), TEXT("ctpCmdTimeout"));
    setPropertyValue(TEXT("SyncClient"), TEXT("ctpCmdTimeout"),    value, true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient")), TEXT("ctpConnTimeout"));
    setPropertyValue(TEXT("SyncClient"), TEXT("ctpConnTimeout"),    value, true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient")), TEXT("ctpServer"));
    setPropertyValue(TEXT("SyncClient"), TEXT("ctpServer"),    value, true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient")), TEXT("ctpPort"));
    setPropertyValue(TEXT("SyncClient"), TEXT("ctpPort"),    value, true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient")), TEXT("notifyTimeout"));
    setPropertyValue(TEXT("SyncClient"), TEXT("notifyTimeout"),    value, true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient")), TEXT("ctpReady"));
    setPropertyValue(TEXT("SyncClient"), TEXT("ctpReady"),    value, true);
    if (value) { delete [] value; value = NULL; }    

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient")), TEXT("askUser"));
    setPropertyValue(TEXT("SyncClient"), TEXT("askUser"),    value, true);
    if (value) { delete [] value; value = NULL; }    

    backupSources();
    backupExtraSources();

    return true;
}

/*
* Take care about the swv property. It is written in the SyncClient key to keep the value of the older
* version installed
*/
bool restoreRegistryVersion() {
    
    
    //deleteAllRegistry(); // Remove %s
    //createAllRegistry();
    wchar_t* value = NULL;

     /*
    * Don't restore the swv version!! copy the value into the SyncClient registry.
    * The right value is in the getJustInsertedRegistry value
    */
    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\syncml\\DevDetail")), TEXT("swv"));
    setPropertyValue(TEXT("SyncClient"), TEXT("oldswv"),    value);
    if (value) { delete [] value; value = NULL; }   

    getJustInsertedRegistry();

    //delete the syncml tree that can have 3.0 references
    deleteSyncmlTree();
    deleteVersion30RegistryValues();
   

    // Auth
    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\syncml\\Auth")), TEXT("username"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Auth"), TEXT("username"),    value);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\syncml\\Auth")), TEXT("password"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Auth"), TEXT("password"),    value);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\syncml\\Auth")), TEXT("clientNonce"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Auth"), TEXT("clientNonce"),    value);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\syncml\\Auth")), TEXT("serverNonce"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Auth"), TEXT("serverNonce"),    value);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\syncml\\Auth")), TEXT("serverID"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Auth"), TEXT("serverID"),    value);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\syncml\\Auth")), TEXT("serverPWD"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Auth"), TEXT("serverPWD"),    value);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\syncml\\Auth")), TEXT("isServerAuthRequired"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Auth"), TEXT("isServerAuthRequired"),    value);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\syncml\\Auth")), TEXT("clientAuthType"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Auth"), TEXT("clientAuthType"),    value);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\syncml\\Auth")), TEXT("serverAuthType"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Auth"), TEXT("serverAuthType"),    value);
    if (value) { delete [] value; value = NULL; }

    // Conn

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\syncml\\Conn")), TEXT("syncUrl"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Conn"), TEXT("syncUrl"),    value);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\syncml\\Conn")), TEXT("proxyHost"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Conn"), TEXT("proxyHost"),    value);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\syncml\\Conn")), TEXT("proxyPort"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Conn"), TEXT("proxyPort"),    value);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\syncml\\Conn")), TEXT("useProxy"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Conn"), TEXT("useProxy"),    value);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\syncml\\Conn")), TEXT("proxyUsername"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Conn"), TEXT("proxyUsername"),    value);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\syncml\\Conn")), TEXT("proxyPassword"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Conn"), TEXT("proxyPassword"),    value);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\syncml\\Conn")), TEXT("checkConn"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Conn"), TEXT("checkConn"),    value);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\syncml\\Conn")), TEXT("responseTimeout"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Conn"), TEXT("responseTimeout"),    value);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\syncml\\Conn")), TEXT("readBufferSize"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Conn"), TEXT("readBufferSize"),    value);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\syncml\\Conn")), TEXT("userAgent"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Conn"), TEXT("userAgent"),    value);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\syncml\\Conn")), TEXT("enableCompression"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Conn"), TEXT("enableCompression"),    value);
    if (value) { delete [] value; value = NULL; }

    // DevInfo

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\syncml\\DevInfo")), TEXT("devID"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\DevInfo"), TEXT("devID"),    value);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\syncml\\DevInfo")), TEXT("dsV"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\DevInfo"), TEXT("dsV"),    value);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\syncml\\DevInfo")), TEXT("man"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\DevInfo"), TEXT("man"),    value);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\syncml\\DevInfo")), TEXT("mod"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\DevInfo"), TEXT("mod"),    value);
    if (value) { delete [] value; value = NULL; }

    //DevDetail
    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\syncml\\DevDetail")), TEXT("devType"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\DevDetail"), TEXT("devType"),    value);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\syncml\\DevDetail")), TEXT("fwv"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\DevDetail"), TEXT("fwv"),    value);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\syncml\\DevDetail")), TEXT("hwv"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\DevDetail"), TEXT("hwv"),    value);
    if (value) { delete [] value; value = NULL; }
   
    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\syncml\\DevDetail")), TEXT("swv"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\DevDetail"), TEXT("swv"),    value);
    if (value) { delete [] value; value = NULL; }   
    
    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\syncml\\DevDetail")), TEXT("oem"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\DevDetail"), TEXT("oem"),    value);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\syncml\\DevDetail")), TEXT("loSupport"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\DevDetail"), TEXT("loSupport"),    value);
    if (value) { delete [] value; value = NULL; }

    // Ext
    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\syncml\\Ext")), TEXT("begin"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Ext"), TEXT("begin"),    value);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\syncml\\Ext")), TEXT("end"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Ext"), TEXT("end"),    value);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\syncml\\Ext")), TEXT("devInfHash"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Ext"), TEXT("devInfHash"),    value);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\syncml\\Ext")), TEXT("firstTimeSyncMode"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Ext"), TEXT("firstTimeSyncMode"),    value);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\syncml\\Ext")), TEXT("logLevel"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Ext"), TEXT("logLevel"),    value);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\syncml\\Ext")), TEXT("maxObjSize"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Ext"), TEXT("maxObjSize"),    value);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\syncml\\Ext")), TEXT("maxMsgSize"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Ext"), TEXT("maxMsgSize"),    value);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\syncml\\Ext")), TEXT("nocSupport"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Ext"), TEXT("nocSupport"),    value);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\syncml\\Ext")), TEXT("utc"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Ext"), TEXT("utc"),    value);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient\\spds\\syncml\\Ext")), TEXT("verDTD"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Ext"), TEXT("verDTD"),    value);
    if (value) { delete [] value; value = NULL; }


   // SyncClient

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient")), TEXT("push"));
    setPropertyValue(TEXT("SyncClient"), TEXT("push"),    value);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient")), TEXT("polling"));
    setPropertyValue(TEXT("SyncClient"), TEXT("polling"),    value);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient")), TEXT("svrNotified"));
    setPropertyValue(TEXT("SyncClient"), TEXT("svrNotified"),    value);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient")), TEXT("pushPort"));
    setPropertyValue(TEXT("SyncClient"), TEXT("pushPort"),    value);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient")), TEXT("sms"));
    setPropertyValue(TEXT("SyncClient"), TEXT("sms"),    value);
    if (value) { delete [] value; value = NULL; }

    // these entries are set by the .inf properties
    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient")), TEXT("pluginVersion"));
    setPropertyValue(TEXT("SyncClient"), TEXT("pluginVersion"),    value);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient")), TEXT("isPortal"));
    setPropertyValue(TEXT("SyncClient"), TEXT("isPortal"),    value);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient")), TEXT("path"));
    setPropertyValue(TEXT("SyncClient"), TEXT("path"),    value);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient")), TEXT("cradleNotif"));
    setPropertyValue(TEXT("SyncClient"), TEXT("cradleNotif"),    value);
    if (value) { delete [] value; value = NULL; }

    /***** restore ctp registry *******/
    
    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient")), TEXT("queuePush"));
    setPropertyValue(TEXT("SyncClient"), TEXT("queuePush"),    value);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient")), TEXT("ctpRetry"));
    setPropertyValue(TEXT("SyncClient"), TEXT("ctpRetry"),    value);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient")), TEXT("maxCtpRetry"));
    setPropertyValue(TEXT("SyncClient"), TEXT("maxCtpRetry"),    value);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient")), TEXT("ctpCmdTimeout"));
    setPropertyValue(TEXT("SyncClient"), TEXT("ctpCmdTimeout"),    value);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient")), TEXT("ctpConnTimeout"));
    setPropertyValue(TEXT("SyncClient"), TEXT("ctpConnTimeout"),    value);
    if (value) { delete [] value; value = NULL; }
    
    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient")), TEXT("ctpServer"));
    setPropertyValue(TEXT("SyncClient"), TEXT("ctpServer"),    value);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient")), TEXT("ctpPort"));
    setPropertyValue(TEXT("SyncClient"), TEXT("ctpPort"),    value);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient")), TEXT("notifyTimeout"));
    setPropertyValue(TEXT("SyncClient"), TEXT("notifyTimeout"),    value);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient")), TEXT("ctpReady"));
    setPropertyValue(TEXT("SyncClient"), TEXT("ctpReady"),    value);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient")), TEXT("askUser"));
    setPropertyValue(TEXT("SyncClient"), TEXT("askUser"),    value);
    if (value) { delete [] value; value = NULL; }
    
    restoreSources();
    restoreExtraSources();

    return true;
}


bool removeBackupRegistryVersion() {

    RegDeleteKey(HKEY_LOCAL_MACHINE,getFullReg(TEXT("SOFTWARE\\%sBkp")));
    return true;
}

void setFrom30() {
    setPropertyValue(TEXT("SyncClient"), TEXT("isFrom30"),    TEXT("true"), true);
}

BOOL isFrom30() {
    wchar_t* value = NULL;
    BOOL ret = FALSE;
    value = readPropertyValue(getFullReg( TEXT("Software\\%sBkp\\SyncClient")), TEXT("isFrom30"));
    if (value && wcscmp(value, TEXT("true")) == 0)
        ret = TRUE;

    if (value)
        delete [] value;
    return ret;
}


bool backupRegistry30() {

    createAllRegistryBackup();

    wchar_t* value = NULL;
    /*
    * first of all check if the current installing version is the same type of the installed one
    * If the installing is portal and the installed not, then reset the username and password and
    * server url with the default. The same if the installed is portal and the installing not.
    * If the type is the same then keep the previous configuration
    */
    bool keepThePreviousConfig = true;
    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient")), TEXT("isPortal"));
    if (value && wcscmp(value, IS_PORTAL) != 0) {  //IS_PORTAL is just put in the bkp registry
        keepThePreviousConfig = false;
    }
    if (value) { delete [] value; value = NULL; }
    // SyncML
    if (keepThePreviousConfig) {
        value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\syncml")), TEXT("username"));
        setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Auth"), TEXT("username"),    value, true);
        if (value) { delete [] value; value = NULL; }

        value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\syncml")), TEXT("password"));
        setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Auth"), TEXT("password"),    value, true);
        if (value) { delete [] value; value = NULL; }

        value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\syncml")), TEXT("syncUrl"));
        setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Conn"), TEXT("syncUrl"),    value,true);
        if (value) { delete [] value; value = NULL; }

    }
    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\syncml")), TEXT("clientNonce"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Auth"), TEXT("clientNonce"),    value, true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\syncml")), TEXT("serverNonce"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Auth"), TEXT("serverNonce"),    value, true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\syncml")), TEXT("serverID"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Auth"), TEXT("serverID"),    value, true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\syncml")), TEXT("serverPWD"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Auth"), TEXT("serverPWD"),    value, true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\syncml")), TEXT("isServerAuthRequired"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Auth"), TEXT("isServerAuthRequired"),    value, true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\syncml")), TEXT("clientAuthType"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Auth"), TEXT("clientAuthType"),    value, true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\syncml")), TEXT("serverAuthType"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Auth"), TEXT("serverAuthType"),    value, true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\syncml")), TEXT("useProxy"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Conn"), TEXT("useProxy"),    value, true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\syncml")), TEXT("enableCompression"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Conn"), TEXT("enableCompression"),    value, true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\syncml")), TEXT("proxyHost"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Conn"), TEXT("proxyHost"),    value, true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\syncml")), TEXT("proxyPort"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Conn"), TEXT("proxyPort"),    value, true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\syncml")), TEXT("checkConn"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Conn"), TEXT("checkConn"),    value, true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\syncml")), TEXT("responseTimeout"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Conn"), TEXT("responseTimeout"),    value, true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\syncml")), TEXT("readBufferSize"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Conn"), TEXT("readBufferSize"),    value, true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\syncml")), TEXT("deviceId"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\DevInfo"), TEXT("devID"),    value, true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\syncml")), TEXT("maxObjSize"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Ext"), TEXT("maxObjSize"),    value, true);
    if (value) { delete [] value; value = NULL; }

    // get the maxMsgSize from the installation procedure and not the previous one
    //value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\syncml"), TEXT("maxMsgSize"));
    //setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Ext"), TEXT("maxMsgSize"),    value, true);
    //if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient\\spds\\syncml")), TEXT("firstTimeSyncMode"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Ext"), TEXT("firstTimeSyncMode"),    value, true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient")), TEXT("logLevel"));
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Ext"), TEXT("logLevel"),    value, true);
    if (value) { delete [] value; value = NULL; }


    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient")), TEXT("path"));
    setPropertyValue(TEXT("SyncClient"), TEXT("path"),    value, true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient")), TEXT("push"));
    setPropertyValue(TEXT("SyncClient"), TEXT("push"),    value, true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient")), TEXT("polling"));
    setPropertyValue(TEXT("SyncClient"), TEXT("polling"),    value, true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient")), TEXT("svrNotified"));
    setPropertyValue(TEXT("SyncClient"), TEXT("svrNotified"),    value, true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient")), TEXT("pushPort"));
    setPropertyValue(TEXT("SyncClient"), TEXT("pushPort"),    value, true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg( TEXT("Software\\%s\\SyncClient")), TEXT("sms"));
    setPropertyValue(TEXT("SyncClient"), TEXT("sms"),    value, true);
    if (value) { delete [] value; value = NULL; }


    backupSources();
    return true;


}
bool removeEntryOfUninstall30(wchar_t* installedName) {
    if (wcslen(installedName) > 0) {
        wchar_t toRemove[256];
        wsprintf(toRemove, TEXT("%s\\%s"), TEXT("SOFTWARE\\Apps"), installedName);
        RegDeleteKey(HKEY_LOCAL_MACHINE, toRemove);
        wsprintf(toRemove, TEXT("%s"), getFullReg(TEXT("SOFTWARE\\Apps\\%s Lib")));
        RegDeleteKey(HKEY_LOCAL_MACHINE, toRemove);


        wsprintf(toRemove, TEXT("%s\\%s"), TEXT("Security\\AppInstall"), installedName);
        RegDeleteKey(HKEY_LOCAL_MACHINE, toRemove);
        wsprintf(toRemove, TEXT("%s"), getFullReg(TEXT("Security\\AppInstall\\%s Lib")));
        RegDeleteKey(HKEY_LOCAL_MACHINE, toRemove);

    }
    return true;
}


void createAllRegistryBackup() {
    createAllRegistry(true);
}

void createAllRegistry(bool backup) {

    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Auth"), TEXT("username"),    USERNAME           ,                           backup);
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Auth"), TEXT("password"),    PASSWORD           ,                           backup);
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Auth"), TEXT("clientNonce"), TEXT("cGlwcG8="                  )           , backup);
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Auth"), TEXT("serverNonce"), TEXT("VGB8YHQ6U25lWmAuR3Z3bw=="  )           , backup);
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Auth"), TEXT("serverID"),    getAppName()                                 , backup); //TEXT("funambol"                  )           , backup);
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Auth"), TEXT("serverPWD"),   getAppName()                                 , backup); //TEXT("funambol"                  )           , backup);
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Auth"), TEXT("isServerAuthRequired"),  TEXT("F"               )           , backup);
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Auth"), TEXT("clientAuthType"), TEXT("syncml:auth-basic"      )           , backup);
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Auth"), TEXT("serverAuthType"), TEXT("syncml:auth-basic"      )           , backup);

    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Conn"), TEXT("syncUrl"  ),  SERVER_URL                                     , backup);
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Conn"), TEXT("proxyHost"),  TEXT(""                                     )  , backup);
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Conn"), TEXT("proxyPort"),  TEXT(""                                     )  , backup);
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Conn"), TEXT("useProxy" ),  TEXT(""                                      ) , backup);
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Conn"), TEXT("proxyUsername"),  TEXT(""                                 )  , backup);
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Conn"), TEXT("proxyPassword"),  TEXT(""                                 )  , backup);
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Conn"), TEXT("checkConn"),  TEXT("1"                                    )  , backup);
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Conn"), TEXT("responseTimeout"), TEXT("0"                              )   , backup);
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Conn"), TEXT("readBufferSize"),  TEXT("4096"                            )  , backup);
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Conn"), TEXT("enableCompression"),  COMPRESSION                            , backup);

    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\DevInfo"), TEXT("devID"),TEXT("" )                                         , backup);
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\DevInfo"), TEXT("dsV"),  TEXT("" )                                         , backup);
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\DevInfo"), TEXT("man"),  TEXT("" )                                         , backup);
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\DevInfo"), TEXT("mod"),  TEXT("" )                                         , backup);

    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\DevDetail"), TEXT("devType"),  TEXT("windowsmobile")                         , backup);
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\DevDetail"), TEXT("fwv"),  TEXT(""               )                         , backup);
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\DevDetail"), TEXT("hwv"),  TEXT(""               )                         , backup);
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\DevDetail"), TEXT("swv"),  TEXT(""               )                         , backup);
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\DevDetail"), TEXT("oem"),  TEXT(""               )                         , backup);
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\DevDetail"), TEXT("loSupport"),  TEXT("1"        )                         , backup);

    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Ext"), TEXT("begin"),  TEXT("0"             )                              , backup);
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Ext"), TEXT("end"  ),  TEXT("0"             )                              , backup);
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Ext"), TEXT("devInfHash"),  TEXT("0"        )                              , backup);
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Ext"), TEXT("firstTimeSyncMode"),  TEXT("0" )                              , backup);
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Ext"), TEXT("logLevel"),  TEXT("1"          )                              , backup);
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Ext"), TEXT("maxObjSize"),  TEXT("2010000"  )                              , backup);
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Ext"), TEXT("maxMsgSize"),  TEXT("65536"    )                              , backup);
    //setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Ext"), TEXT("maxMsgSize"),  TEXT("16384"    )                              , backup);
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Ext"), TEXT("nocSupport"),  TEXT("1"        )                              , backup);
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Ext"), TEXT("utc"),  TEXT("0"               )                              , backup);
    setPropertyValue(TEXT("SyncClient\\spds\\syncml\\Ext"), TEXT("verDTD"),  TEXT("1.1"          )                              , backup);

    setPropertyValue(TEXT("SyncClient\\spds\\sources\\contacts"), TEXT("sync"),  TEXT("two-way"                  )              , backup);
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\contacts"), TEXT("last"),  TEXT("0"                        )              , backup);
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\contacts"), TEXT("name"),  TEXT("contact"                  )              , backup);
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\contacts"), TEXT("type"),  TEXT("text/x-s4j-sifc"          )              , backup);
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\contacts"), TEXT("syncModes"),  TEXT("slow,two-way,refresh")              , backup);
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\contacts"), TEXT("uri"),  TEXT("oxscard"                     )              , backup);
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\contacts"), TEXT("useSIF"),  TEXT("1"                      )              , backup);
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\contacts"), TEXT("encoding"),  TEXT("b64"                  )              , backup);
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\contacts"), TEXT("schedule"),  TEXT("0"                    )              , backup);
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\contacts"), TEXT("encryption"),  TEXT("0"                  )              , backup);

    setPropertyValue(TEXT("SyncClient\\spds\\sources\\appointments"), TEXT("sync"),  TEXT("two-way"                               ), backup);
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\appointments"), TEXT("last"),  TEXT("0"                                     ), backup);
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\appointments"), TEXT("name"),  TEXT("calendar"                              ), backup);
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\appointments"), TEXT("type"),  TEXT("text/x-s4j-sife"                       ), backup);
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\appointments"), TEXT("syncModes"),  TEXT("slow,two-way,refresh"             ), backup);
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\appointments"), TEXT("uri"),  TEXT("oxscal"                                   ), backup);
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\appointments"), TEXT("useSIF"),  TEXT("1"                                   ), backup);
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\appointments"), TEXT("encoding"), TEXT("b64"                                ), backup);
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\appointments"), TEXT("schedule"),  TEXT("0"                                 ), backup);
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\appointments"), TEXT("encryption"), TEXT("0"                                ), backup);

    setPropertyValue(TEXT("SyncClient\\spds\\sources\\tasks"), TEXT("sync"),  TEXT("two-way"                                         ), backup);
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\tasks"), TEXT("last"),  TEXT("0"                                            ), backup);
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\tasks"), TEXT("name"),  TEXT("task"                                         ), backup);
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\tasks"), TEXT("type"),  TEXT("text/x-s4j-sift"                              ), backup);
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\tasks"), TEXT("syncModes"),  TEXT("slow,two-way,refresh"                    ), backup);
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\tasks"), TEXT("uri"),  TEXT("oxstask"                                         ), backup);
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\tasks"), TEXT("useSIF"), TEXT("1"                                           ), backup);
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\tasks"), TEXT("encoding"), TEXT("b64"                                       ), backup);
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\tasks"), TEXT("schedule"), TEXT("0"                                         ), backup);
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\tasks"), TEXT("encryption"),  TEXT("0"                                      ), backup);

    // default dir directory is NOT empty (form v6) for the first installation. If it is empty it is possible
    // to set the default value from the UI. Otherwise the default is always  \My Documents
    wchar_t* defaultDir = getMyDocumentFolder();
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\notes"), TEXT("sync"),  TEXT("none"                                        ), backup);
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\notes"), TEXT("last"),  TEXT("0"                                           ), backup);
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\notes"), TEXT("name"),  TEXT("note"                                        ), backup);
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\notes"), TEXT("type"),  TEXT("text/x-s4j-sifn"                             ), backup);
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\notes"), TEXT("syncModes"),  TEXT("slow,two-way,refresh"                   ), backup);
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\notes"), TEXT("uri"),  TEXT("snote"                                        ), backup);
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\notes"), TEXT("useSIF"),  TEXT("1"                                         ), backup);
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\notes"), TEXT("encoding"),  TEXT("b64"                                     ), backup);
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\notes"), TEXT("schedule"),  TEXT("0"                                       ), backup);
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\notes"), TEXT("encryption"),  TEXT("0"                                     ), backup);
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\notes"), TEXT("dir"),  defaultDir                                           , backup);

    setPropertyValue(TEXT("SyncClient\\spds\\sources\\briefcase"), TEXT("sync"),  TEXT("none"                                    ), backup);
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\briefcase"), TEXT("last"),  TEXT("0"                                       ), backup);
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\briefcase"), TEXT("name"),  TEXT("briefcase"                               ), backup);
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\briefcase"), TEXT("type"),  TEXT("application/*"                           ), backup);
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\briefcase"), TEXT("syncModes"),  TEXT("slow,two-way,refresh"               ), backup);
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\briefcase"), TEXT("uri"),  TEXT("briefcase"                                ), backup);
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\briefcase"), TEXT("encoding"),  TEXT("bin"                                 ), backup);
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\briefcase"), TEXT("dir"), defaultDir                                        , backup);
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\briefcase"), TEXT("schedule"),  TEXT("0"                                   ), backup);
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\briefcase"), TEXT("encryption"),  TEXT("0"                                 ), backup);

    if (defaultDir) { delete [] defaultDir; }

    setPropertyValue(TEXT("SyncClient\\spds\\sources\\mails"), TEXT("sync"),  TEXT("two-way"                                      ), backup);
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\mails"), TEXT("last"),  TEXT("0"                                            ), backup);
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\mails"), TEXT("name"),  TEXT("mail"                                         ), backup);
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\mails"), TEXT("type"),  TEXT("application/vnd.omads-email+xml"              ), backup);
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\mails"), TEXT("syncModes"),  TEXT("slow,two-way,refresh"                    ), backup);
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\mails"), TEXT("uri"),  TEXT("mail"                                          ), backup);
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\mails"), TEXT("useSIF"),  TEXT("0"                                          ), backup);
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\mails"), TEXT("encoding"),  TEXT("bin"                                      ), backup);
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\mails"), TEXT("Inbox"),  TEXT("1"                                           ), backup);
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\mails"), TEXT("Outbox"),  TEXT("1"                                          ), backup);
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\mails"), TEXT("Trash"),  TEXT("0"                                           ), backup);
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\mails"), TEXT("Sent"),  TEXT("0"                                            ), backup);
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\mails"), TEXT("Draft"),  TEXT("0"                                           ), backup);
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\mails"), TEXT("attachSize"),  TEXT("10"                                     ), backup);
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\mails"), TEXT("bodySize"),  TEXT("1"                                        ), backup);
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\mails"), TEXT("downloadAge"),  TEXT("3"                                     ), backup);
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\mails"), TEXT("schedule"),  TEXT("0"                                        ), backup);
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\mails"), TEXT("mailMaxMsgSize"),  TEXT("524288"                             ), backup);
    setPropertyValue(TEXT("SyncClient\\spds\\sources\\mails"), TEXT("encryption"),  TEXT("0"                                      ), backup);

    setPropertyValue(TEXT("SyncClient"), TEXT("svrNotified"),  TEXT("0"                                                           ), backup);
    setPropertyValue(TEXT("SyncClient"), TEXT("polling"),  TEXT("0"                                                               ), backup);
    setPropertyValue(TEXT("SyncClient"), TEXT("pushPort"),  TEXT("4745"                                                           ), backup);
    setPropertyValue(TEXT("SyncClient"), TEXT("sms"),  TEXT("1"                                                                   ), backup);
    setPropertyValue(TEXT("SyncClient"), TEXT("push"),  TEXT("1"                                                                  ), backup);
    setPropertyValue(TEXT("SyncClient"), TEXT("isPortal"),  IS_PORTAL                                                              , backup);


    setPropertyValue(TEXT("SyncClient"), TEXT("path"),  TEXT("\\"                                                                 ), backup);
    setPropertyValue(TEXT("SyncClient"), TEXT("cradleNotif"),  TEXT("0"                                                           ), backup);

    // **** CTP new properties (push and polling are already inserted
    setPropertyValue(TEXT("SyncClient"), TEXT("queuePush"),        TEXT("1"                                                       ), backup);
    setPropertyValue(TEXT("SyncClient"), TEXT("queueList"),        TEXT("0"                                                       ), backup);
    setPropertyValue(TEXT("SyncClient"), TEXT("ctpRetry"),         TEXT("5"                                                       ), backup);
    setPropertyValue(TEXT("SyncClient"), TEXT("maxCtpRetry"),      TEXT("900"                                                     ), backup);
    setPropertyValue(TEXT("SyncClient"), TEXT("ctpCmdTimeout"),    TEXT("60"                                                      ), backup);
    setPropertyValue(TEXT("SyncClient"), TEXT("ctpConnTimeout"),   TEXT("0"                                                       ), backup);
    setPropertyValue(TEXT("SyncClient"), TEXT("ctpServer"),        TEXT(""                                                        ), backup);
    setPropertyValue(TEXT("SyncClient"), TEXT("ctpPort"),          TEXT("4745"                                                    ), backup);
    setPropertyValue(TEXT("SyncClient"), TEXT("ctpReady"),         TEXT("300"                                                     ), backup);
    setPropertyValue(TEXT("SyncClient"), TEXT("notifyTimeout"),    TEXT("120"                                                     ), backup);

    setPropertyValue(TEXT("SyncClient"), TEXT("askUser"),          TEXT("0"                                                       ), backup);

}


/**
 * Rewrite the entries that we want to change during update.
 * These registry entries existed already in the last release, but we
 * want a different value now.
*/ 
void updateNewRegistry() {
    // none
}






wchar_t* getMyDocumentFolder() {

    wchar_t* dir = new wchar_t[256];
    SHGetSpecialFolderPath(NULL, dir, CSIDL_PERSONAL, FALSE);
    return dir;
}

void setPropertyValue(const wchar_t*  context, const wchar_t*  prop, const wchar_t* value, bool backup) {
    if ((prop == NULL) || (value == NULL)) {
        return;
    }
    wchar_t fullContext[256];
    

    if (backup)
        wsprintf(fullContext, TEXT("%s\\%s"), getFullReg( TEXT("Software\\%sBkp")), context);
    else
        wsprintf(fullContext, TEXT("%s\\%s"), getFullReg( TEXT("Software\\%s")), context);

   // MessageBox(NULL, fullContext,  getAppName(),  MB_SETFOREGROUND | MB_ICONHAND | MB_OK);


    HKEY key;
    DWORD res;

    RegCreateKeyEx(
            HKEY_LOCAL_MACHINE,
            fullContext,
            0,
            NULL,
            REG_OPTION_NON_VOLATILE,
            KEY_ALL_ACCESS,
            NULL,
            &key,
            &res
            );


    if (key == 0) {
        goto finally;
    }


    RegSetValueEx(
            key,
            prop,
            NULL,
            REG_SZ,  // we currently support only strings
            (UCHAR*)value,
            (wcslen(value)+1)*sizeof(WCHAR)
            );


    RegFlushKey(key);
    //RegCloseKey(key);

finally:

    if (key != 0) {
        RegCloseKey(key);
    }
}


wchar_t* readPropertyValue(const wchar_t* fullContext, const wchar_t*  prop) {
    HKEY key = NULL;
    DWORD res;
    long err = 0;
    wchar_t* buf = NULL;
    ULONG dim = 0;

    RegCreateKeyEx(
            HKEY_LOCAL_MACHINE,
            fullContext,
            0,
            NULL,
            REG_OPTION_NON_VOLATILE,
            KEY_ALL_ACCESS,
            NULL,
            &key,
            &res
            );

    if (key == 0) {
        goto finally;
    }

    // Get value length
    err = RegQueryValueEx(
            key,
            prop,
            NULL,
            NULL,  // we currently support only strings
            NULL,
            &dim
            );

    if (err == ERROR_SUCCESS) {
        if (dim > 0) {
            buf = new wchar_t[dim + 1];

            err = RegQueryValueEx(
                    key,
                    prop,
                    NULL,
                    NULL,  // we currently support only strings
                    (UCHAR*)buf,
                    &dim
                    );
        }
    }
    //else MessageBox(NULL,  "Error", "getConfigParameter", MB_OK);
finally:

    if (key != 0) {
        RegCloseKey(key);
    }

    return buf;
}



/*
 * Returns the children's name of the parent node.
 *
 * @param children - the buffer where ManagementNode* must be stored
 * @param size - the size of the children buffer (number of ManagementNode*) in
 *               input; the number of children in output
 */
wchar_t** getChildrenNames() {
    ULONG dim = 0;
    HKEY key;
    wchar_t **childrenName ;
    ULONG i, howMany = 0;
    const wchar_t fullContext[] = TEXT("Security\\AppInstall");

    DWORD res;
    HRESULT ret;

    RegCreateKeyEx(
            HKEY_LOCAL_MACHINE,
            fullContext,
            0,
            NULL,
            REG_OPTION_NON_VOLATILE,
            KEY_ALL_ACCESS,
            NULL,
            &key,
            &res
            );

    if (key == 0) {
        MessageBox (NULL, TEXT("Impossible to read the program installation registry. \
                                Maybe the application has not proviliged access"), TEXT ("Alert"), MB_SETFOREGROUND | MB_ICONHAND | MB_OK);
        goto finally;
    }
    //
    // First of all we need to know how many
    RegQueryInfoKey(
            key, NULL, NULL, NULL, &howMany, NULL, NULL, NULL,
            NULL, NULL, NULL, NULL
            );

    if (howMany == 0)
        return NULL;

    childrenName = new wchar_t *[howMany + 1];

    for (i=0; i<howMany; i++) {
        dim = 256*sizeof(wchar_t);
        TCHAR child[256] = TEXT("");

        ret = RegEnumKeyEx(key, i, child, &dim, NULL, NULL, NULL, NULL);

        if (ret != ERROR_SUCCESS) {
            if (ret == ERROR_NO_MORE_ITEMS) {
                break;
            }
            else {
                MessageBox (NULL, TEXT("Impossible to count the program installation nodes. \
                                Maybe the application has not proviliged access"), TEXT ("Alert"), MB_SETFOREGROUND | MB_ICONHAND | MB_OK);
                goto finally;
            }
        }
        childrenName[i] = new wchar_t[256];
        wcscpy(childrenName[i], child);
    }
    childrenName[howMany] = NULL;

finally:

    if (key != 0) {
        RegCloseKey(key);
    }

    return childrenName;
}

