/*
 * Copyright (C) 2003-2007 Funambol, Inc
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY, TITLE, NONINFRINGEMENT or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 * 02111-1307  USA
 */

#ifndef INCL_CTP_MANAGER
#define INCL_CTP_MANAGER

/** @cond DEV */

#include <stdio.h>
#include <windows.h>    
#include "CTPMessage.h"
#include "CTPConfig.h"
#include "worker.h"


#define CTP_PROTOCOL_VERSION            0x10        /**< CTP Protocol version = 1.0 */
#define CTP_RETRY_INCREASE_FACTOR       2           /**< Each time the CTP connection is broken, we double the ctpRetry time */


/**
 * The state of CTP connection.
 * State CTP_STATE_WAITING_RESPONSE is used to check the timeout (ctpCmdTimeout)
 * for which the client shall wait for the server response, after sending a command.
 */
typedef enum {
    CTP_STATE_DISCONNECTED          = 0, 
    CTP_STATE_CONNECTING            = 1, 
    CTP_STATE_CONNECTED             = 2,
    CTP_STATE_AUTHENTICATING        = 3, 
    CTP_STATE_READY                 = 4, 
    CTP_STATE_WAITING_RESPONSE      = 5, 
    CTP_STATE_CLOSING               = 6
} CtpState;



/**
 * The class to manage the CTP connection, singleton.
 */
class CTPManager {

private:

    /// Pointer to CTPManager instance
    static CTPManager* pinstance;


    /// The configuration of CTP
    CTPConfig config;

    /// The state of CTP connection.
    CtpState ctpState;

    /// Used by threads to exit in a soft way.
    bool leaving;

    /// The socket used
    SOCKET ctpSocket;

    HANDLE ctpThread;                   /**< Handle of main CTP thread, which implements the CTP process */
    HANDLE receiveThread;               /**< Handle of thread used to receive msg from Server */
    HANDLE heartbeatThread;             /**< Handle of thread used to send ready msg to Server */
    HANDLE cmdTimeoutThread;            /**< Handle of thread used to check if a response arrived in ctpCmdTimeout seconds */


    /// Store the received message from Server
    CTPMessage* receivedMsg;

    // For debugging
    int totalBytesSent;
    int totalBytesReceived;

    // Private methods:
    int sendMsg(CTPMessage* message);
    string createMD5Credentials();
    char* createErrorMsg(DWORD errorCode = 0);


protected:

    // Constructor
    CTPManager();


public:

    // Method to get the sole instance of CTPManager
    static CTPManager* getInstance();

    ~CTPManager();

    HANDLE startCTP(HANDLE stpThread);
    int stopCTP();
    int openConnection();
    int closeConnection();
    int receive();
    bool stopThread(HANDLE thread, DWORD exitcode = 0);

    // Create and send messages through the socket.
    int sendReadyMsg();
    int sendAuthMsg();
    int sendByeMsg();

    CTPMessage* receiveStatusMsg();

    /**
     * Returns pointer to (internally owned) CTPConfig
     */
    CTPConfig* getConfig() { return &config; }



    /// Get the ctpState member.
    CtpState getCtpState()    { return ctpState; }

    /// Set the ctpState member.
    void setCtpState(CtpState v) { ctpState = v; }

    /// Returns true if CTP is leaving.
    bool isLeaving() { return leaving; }
};


/** @endcond */
#endif

