/*
 * Copyright (C) 2003-2007 Funambol, Inc
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY, TITLE, NONINFRINGEMENT or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 * 02111-1307  USA
 */

#include <windows.h>
#include <initguid.h>   
#include <pimstore.h>   
#include <aygshell.h>   
#include <commctrl.h>   

#include "ClientPush.h"   
#include "spdm/DMTree.h"
#include "spdm/DMTreeFactory.h"
#include "pim/Utils.h"
#include "base/util/utils.h"

#include <Pmpolicy.h>
#include <Pm.h>
#include <string>
#include <list>

// Global Variables
HINSTANCE                cp_hInstanc  = 0;
HWND                     cp_hWnd      = 0;
WNDCLASS                 cp_wncClass;

static IPOutlookApp2            *g_polApp   = NULL;
static IPOutlookItemCollection  *g_pCol     = NULL;
static const LPWSTR g_szClassName           = L"ClientPushWindow2";

DWORD WINAPI initClientPush(LPVOID moduleHandle) {
    
    MSG         msg;
    int         rc = 0;
    HRESULT     hr = S_OK;

    // Raise thread priority because we need to keep the thread working costantly
    SetThreadPriority(GetCurrentThread(), THREAD_PRIORITY_HIGHEST);
    PowerPolicyNotify(PPN_UNATTENDEDMODE, TRUE);
    
    try {
        // Initialize application
        rc = createClientPushClass(&cp_wncClass, (HANDLE)moduleHandle);
        if (rc < 0) {
            LOG.error("CP: Error in creating the push class. Exit...");        
            return rc;
        }
        LOG.info("CP: Client push class created");        
        // Init POOM

        LOG.debug("Init Poom");
        hr = InitPOOM2();
        CHR(hr);
        
        LOG.debug("CP: Log on the IPOutlookApp2 interface");    
        hr = g_polApp->Logon((long)cp_hWnd); 
        CHR(hr);
        LOG.debug("CP: Subscription of Contact folder...");
        SubscribeToNotifications(g_polApp, olContactItem, PIMFOLDERNOTIFICATION_REMOTE);
        if (hr == S_OK) {
            LOG.info("CP: Contact folder subscribed.");
        }
        LOG.debug("CP: Subscription of Calendar folder...");
        SubscribeToNotifications(g_polApp, olAppointmentItem, PIMFOLDERNOTIFICATION_REMOTE);
        if (hr == S_OK) {
            LOG.info("CP: Calendar folder subscribed.");
        }
        LOG.info("CP: Subscription of Task folder...");
        SubscribeToNotifications(g_polApp, olTaskItem, PIMFOLDERNOTIFICATION_REMOTE);
        if (hr == S_OK) {
            LOG.info("CP: Task folder subscribed.");
        }                
        LOG.info("CP: Init waiting PIM notification messages...");
        // Main message loop
        while (GetMessage(&msg, NULL, 0, 0)) 
        {
            TranslateMessage(&msg);
            DispatchMessage(&msg);
        }
    } catch(...) {
        LOG.error("CP: Generic error in client Push: Error = 0x%x", GetLastError());
    }

Error:
       
    // UnInit POOM
    UnInitPOOM2();      
    UnregisterClass(cp_wncClass.lpszClassName, cp_wncClass.hInstance);    

    PowerPolicyNotify(PPN_UNATTENDEDMODE, FALSE);
    return 0;
}


///////////////////////////////////////////////////////////////////////////////
// Function:
//      InitApp
//
// Purpose:
//      Checks if the app dialog class already exists. If so, it means the app 
//      is already running so we quit this instance.
//
// Parameters:
//      hInstance       - Handle to current app instance
//      lpWndClass      - pointer to WNDCLASS struct which is filled with app settings.
//
// Returns:
//      0   - Success.
//      -1  - Failure.
//
int createClientPushClass (WNDCLASS *lpWndClass, HANDLE module) {
    int ret = 0;           

    lpWndClass->style                = 0;
    lpWndClass->lpfnWndProc          = (WNDPROC) wndMainProc;
    lpWndClass->cbClsExtra           = 0;
    lpWndClass->cbWndExtra           = 0;
    lpWndClass->hInstance            = (HINSTANCE)module;
    lpWndClass->hIcon                = NULL;
    lpWndClass->hCursor              = 0;
    lpWndClass->hbrBackground        = (HBRUSH) GetStockObject(WHITE_BRUSH);
    lpWndClass->lpszMenuName         = NULL;
    lpWndClass->lpszClassName        = g_szClassName;
    
    DWORD dwError = 0;
    SetLastError(0);
    if (RegisterClass(lpWndClass) == 0) {
        
        dwError = GetLastError();
        if (dwError == ERROR_CLASS_ALREADY_EXISTS) {
            LOG.error("Failed to registed class. Error = 0x%x", dwError);
            ret = -2;            
            goto Error;
        }
        ret = -1; // generic error        
    } else { // success
        dwError = GetLastError();        
        
        cp_hWnd = CreateWindow(
                        g_szClassName,
                        NULL,
                        WS_BORDER,
                        0,0,
                        0,0,
                        NULL,
                        NULL,
                        (HINSTANCE)module,
                        NULL);
    }        

Error:    
       
    return ret;
}




///////////////////////////////////////////////////////////////////////////////
// Function:
//      DlgMainProc
//
//  Process WM messages to the dialog box. 
//
// Purpose:
//      Checks if the app dialog class already exists. If so, it means the app 
//      is already running so we quit this instance.
//
// Parameters:
//      hDlg        - [IN] Handle to main dialog 
//      message     - [IN] WM message
//      wParam      - [IN] WPARAM for WM message
//      lParam      - [IN] LPARAM for WM message. It is the folder when the
//                         PIM_ITEM_ is invoked
//
// Returns:
//      An LRESULT
//
LRESULT CALLBACK wndMainProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam) 
{
    HRESULT     hr  = S_OK;    
    static HWND hWndStats;
    WORD        wHi = HIWORD(wParam), 
                wLo = LOWORD(wParam);

    // Keep this static so we reuse the same variable each time this function is called.
    static IPOutlookItemCollection *pCol;

    CBR(hwnd);
           
    switch (message) 
    {
        case WM_CLOSE:
            LOG.info("CP: WM_CLOSE the application");
            UnInitPOOM2();
            LOG.debug("Unregister class");
            UnregisterClass(cp_wncClass.lpszClassName, cp_wncClass.hInstance);
            LOG.debug("End dialog");
            EndDialog(hwnd, 0);
            LOG.debug("Destroy wnd");
            DestroyWindow (hwnd);
            PowerPolicyNotify(PPN_UNATTENDEDMODE, FALSE);
            break;
            
        case WM_DESTROY:
            UnInitPOOM2();
            PostQuitMessage(0);
            break;      

        case PIM_ITEM_CREATED_LOCAL:
        case PIM_ITEM_DELETED_LOCAL:
        case PIM_ITEM_CHANGED_LOCAL:
            // no modification in local
            break;
        case PIM_ITEM_CREATED_REMOTE:
        case PIM_ITEM_DELETED_REMOTE:
        case PIM_ITEM_CHANGED_REMOTE:            
            setPIMModifications(lParam);
            LOG.info("CP: Item is %lu", wParam);
            // Caught a POOM notification.
            
            CHR(hr);
            break;

        default:
            return DefWindowProc(hwnd, message, wParam, lParam);
    }

Error:
   
    return FALSE;
}


///////////////////////////////////////////////////////////////////////////////
// Function:
//      ItemTypeToFolder
//
// Purpose:
//      Returns the Outlook folder type given the Outlook Item type.
//
// Parameters:
//      olItemType - Outlook item type
//
// Returns:
//      An OlDefaultFolders to the specified Item Type
//
OlDefaultFolders ItemTypeToFolder(int olItemType)
{
    OlDefaultFolders olFolder = (OlDefaultFolders) 0;
    
    switch(olItemType)
    {
        case olAppointmentItem:
            olFolder = olFolderCalendar;
            break;
            
        case olContactItem:
            olFolder = olFolderContacts;
            break;

        case olTaskItem:
            olFolder = olFolderTasks;
            break;
            
        default:
            ASSERT(FALSE);
    }

    return olFolder;
}


///////////////////////////////////////////////////////////////////////////////
// Function:
//      SubscribeToNotifications
//
// Purpose:
//      This function is used to set the level of notification the current process
//      want to get from POOM.  The options include any one of the three values:
//      PIMFOLDERNOTIFICATION_REMOTE, PIMFOLDERNOTIFICATION_LOCAL,
//      or PIMFOLDERNOTIFICATION_ALL
//
// Parameters:
//      pPoom                   - [IN] ptr to POOM app.
//      olItemType              - [IN] Item type.
//      uiNotificationsType     - [IN] Type of notification, remote or local?
//
// Returns:
//      An HRESULT
//
HRESULT SubscribeToNotifications(IPOutlookApp2 *pPoom, int olItemType, uint uiNotificationsType)
{
    HRESULT     hr          = 0;
    IFolder     *pFolder    = NULL;
    IItem       *pItem      = NULL;
    CEPROPVAL   propval     = {0};

    // Parameter check
    CBR(NULL != pPoom);
    if(!uiNotificationsType)
    {
        CHR(E_INVALIDARG);
    }

    // Get the folder for the item type.
    hr = pPoom->GetDefaultFolder(ItemTypeToFolder(olItemType), &pFolder);
    CHR(hr);
    CBR(NULL != pFolder);
    
    hr = pFolder->QueryInterface(IID_IItem, (LPVOID*)&pItem);
    CHR(hr);
    CBR(NULL != pItem);

    // Set the props on the folder.
    propval.propid      = PIMPR_FOLDERNOTIFICATIONS;
    propval.val.ulVal   = uiNotificationsType;
    hr = pItem->SetProps(0, 1, &propval);
    CHR(hr);
        
Error:
    RELEASE_OBJ(pItem);
    RELEASE_OBJ(pFolder);
    return hr;    
}


///////////////////////////////////////////////////////////////////////////////
// Function:
//      InitPOOM2
//
// Purpose:
//      Initilizes COM, creates the POOM COM object and calls Logon.
//
// Parameters:
//      void
//
// Returns:
//      An HRESULT
//
HRESULT InitPOOM2()
{
    HRESULT hr = S_OK;
    
    CHR(CoInitializeEx(NULL, 0));
      
    hr = CoCreateInstance(CLSID_Application, NULL, CLSCTX_INPROC_SERVER, 
        IID_IPOutlookApp2, (LPVOID *) &g_polApp);
    CHR(hr);
    CBR(NULL != g_polApp);

Error:        
    return hr;
}


///////////////////////////////////////////////////////////////////////////////
// Function:
//      UnInitPOOM2
//
// Purpose:
//      Does a logoff on POOM, and un-initilazize COM.
//
// Parameters:
//      void
//
// Returns:
//      An HRESULT
//
HRESULT UnInitPOOM2()
{
    HRESULT hr = S_OK;

    // Lets release the global ptr
    RELEASE_OBJ(g_pCol);
    
    CBR(g_polApp != NULL);

    // This logoff is for safety. (if its already logged off, S_FALSE will get returned)
    hr = g_polApp->Logoff();
    
Error:    
    RELEASE_OBJ(g_polApp);    
    CoUninitialize();   
    return hr;
}

void setPIMModifications(int olFolderType) {
    
    DMTree* dmt          = NULL;
    ManagementNode* node = NULL;
    char* value          = NULL;
    int mask = 0;
    char buf[4];
    
    dmt = DMTreeFactory::getDMTree(APPLICATION_URI);
    if (!dmt)   goto finally;
    node = dmt->readManagementNode(APPLICATION_URI);
    if (!node)  goto finally;
        
    // Check if we use the queue (queuePush = "1")
    value = node->readPropertyValue(CP_PUSH_LIST);
    
    if (!value) {
        value = stringdup("0");
    }
    mask = atoi(value);
    
    //
    // Update the bit-mask with current sources
    //   
    switch(olFolderType)
    {
        case olFolderContacts:
            mask = mask | 2;
            break;
        
        case olFolderCalendar:
            mask = mask | 4;
            break;            
        
        case olFolderTasks:
            mask = mask | 8;
            break;
            
        default:
            break;
    }
   
    // Write 'clientPushList' value to registry
    sprintf(buf, "%d", mask);
    node->setPropertyValue(CP_PUSH_LIST, buf);    

finally:
    if (dmt)   delete dmt;
    if (node)  delete node;
    if (value) delete [] value;

}

