/*
 * Copyright (C) 2003-2007 Funambol, Inc
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY, TITLE, NONINFRINGEMENT or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 * 02111-1307  USA
 */

#ifndef INCL_CLIENT_PUSH
    #define INCL_CLIENT_PUSH
/** @cond DEV */
    
    #include "base/Log.h"
    #include "pim/maincpp.h"
    
    #define MAX_LOADSTRING  255

    #define _ErrorLabel Error

    #define CHR(hResult) \
        if(FAILED(hResult)) { hr = (hResult); goto _ErrorLabel;} 

    #define CBR(fBool) \
        if(!(fBool)) { hr = (E_FAIL); goto _ErrorLabel;} 

    #define RELEASE_OBJ(s)  \
        if (s != NULL)      \
        {                   \
            s->Release();   \
            s = NULL;       \
        }

    // Forward function declarations

    /**
    * Return the proper PIM folder given the item type.
    */
    OlDefaultFolders ItemTypeToFolder(int olItemType);

    /**
    * Callback that is called by the OS to notify the PIM changes.
    * It writes the folder modified in the 'clientPushList' key
    */ 
    LRESULT CALLBACK wndMainProc(HWND, UINT, WPARAM, LPARAM);
    
    /**
    * Create the client push class and the window to receive the messages
    */
    int createClientPushClass (WNDCLASS *lpWndClass, HANDLE moduleHandle);
    
    /**
    * This function is used to set the level of notification the current dll
    * want to get from POOM. It uses the PIMFOLDERNOTIFICATION_REMOTE option.
    */
    HRESULT SubscribeToNotifications(IPOutlookApp2 *pPoom, int olItemType, uint uiNotificationsType);
    
    /**
    * Init the IpOutlookApp2.
    */
    HRESULT InitPOOM2();
    
    /**
    * Release the IpOutlookApp2.
    */
    HRESULT UnInitPOOM2();
    
    /**
    * Function that init the client push mechanism. It creates its own class and
    * the hwnd to listen to the messages sent by the OS about the contacts, calendar and 
    * task modification
    */
    DWORD WINAPI initClientPush(LPVOID moduleHandle);

    /**
    * Reads and updates the property 'clientPushList' from Win registry with
    * the list of sources passed as parameter.
    * The 'clientPushList' value is a bitmask value used to identify which sources
    * have been modified and synced after a predefined scheduled time (5 mins). 
    * The 'clientPushList' bitmask is formatted as:
    *      bit 1  ->  contact
    *      bit 2  ->  calendar
    *      bit 3  ->  task
    *
    * @param olItemType  the PIM item type 
    */
    void setPIMModifications(int olItemType);

   
    
/** @endcond */
#endif
